'use strict';

exports.__esModule = true;

var _alt$custom$lazy$match$seq$str = require('parsly/parser');

function isWhitespace(ch) {
  return ch === 32 || ch >= 9 && ch <= 13;
}

function isIdentifierStart(ch) {
  return ch === 36 || ch === 95 || ch >= 65 && ch <= 90 || ch >= 97 && ch <= 122;
}

function isIdentifierChar(ch) {
  return ch === 36 || ch === 95 || ch >= 65 && ch <= 90 || ch >= 97 && ch <= 122 || ch >= 48 && ch <= 57;
}

// Custom parsers & helpers
// ------------------------

/**
 * Matches whitespaces.
 */
var ws = _alt$custom$lazy$match$seq$str.custom(function (resolve, reject) {
  return function (stream, index) {
    while (isWhitespace(stream.charCodeAt(index))) {
      index++;
    }

    return resolve(index);
  };
});

/**
 * Creates a new parser that skips whitespaces before parsing the token.
 */
var lex = function lex(atom) {
  return _alt$custom$lazy$match$seq$str.custom(function (resolve, reject) {
    return function (stream, index) {
      var result = ws.run(stream, index);
      return atom.run(stream, result.index).aggregate(result);
    };
  });
};

/**
 * Matches identifiers.
 * The implementation does not handle astal range.
 */
var ident = lex(_alt$custom$lazy$match$seq$str.custom(function (resolve, reject) {
  return function (stream, index) {
    if (isIdentifierStart(stream.charCodeAt(index))) {
      var start = index;
      while (isIdentifierChar(stream.charCodeAt(index))) {
        index++;
      }

      return resolve(index, stream.slice(start, index));
    }

    return reject(index, 'Expected valid identifier');
  };
}));

/**
 * Creates a parser that match `glue` separated tokens.
 * The implementation is not fully compliant with JS calls.
 *
 *    ()          // ok
 *    (a, b, c)   // ok
 *    (a, b, c,)  // inconsistency
 *    (a b c)     // inconsistency
 */
var sep = function sep(atom, glue) {
  return _alt$custom$lazy$match$seq$str.seq(atom, glue.maybe()).many();
};

// Punctuations
// ------------

var braceL = lex(_alt$custom$lazy$match$seq$str.str('{'));
var braceR = lex(_alt$custom$lazy$match$seq$str.str('}'));
var parenL = lex(_alt$custom$lazy$match$seq$str.str('('));
var parenR = lex(_alt$custom$lazy$match$seq$str.str(')'));
var angleL = lex(_alt$custom$lazy$match$seq$str.str('<'));
var angleR = lex(_alt$custom$lazy$match$seq$str.str('>'));
var period = lex(_alt$custom$lazy$match$seq$str.str('.'));
var comma = lex(_alt$custom$lazy$match$seq$str.str(','));
var amp = lex(_alt$custom$lazy$match$seq$str.str('&'));

// Naive JSON parser
// -----------------

var number = _alt$custom$lazy$match$seq$str.match(/-?(?:0|[1-9]\d*)(?:\.\d+)?(?:[eE][+-]?\d+)?/);
var string = _alt$custom$lazy$match$seq$str.seq(_alt$custom$lazy$match$seq$str.str('"'), _alt$custom$lazy$match$seq$str.match(/[^"]+/), _alt$custom$lazy$match$seq$str.str('"'));
var array = _alt$custom$lazy$match$seq$str.lazy(function () {
  return _alt$custom$lazy$match$seq$str.seq(_alt$custom$lazy$match$seq$str.str('['), _alt$custom$lazy$match$seq$str.alt(array, _alt$custom$lazy$match$seq$str.match(/[^\[\]]+/)).many(), _alt$custom$lazy$match$seq$str.str(']'));
});
var object = _alt$custom$lazy$match$seq$str.lazy(function () {
  return _alt$custom$lazy$match$seq$str.seq(_alt$custom$lazy$match$seq$str.str('{'), _alt$custom$lazy$match$seq$str.alt(object, _alt$custom$lazy$match$seq$str.match(/[^\{\}]+/)).many(), _alt$custom$lazy$match$seq$str.str('}'));
});
var keywords = _alt$custom$lazy$match$seq$str.alt(_alt$custom$lazy$match$seq$str.str('null'), _alt$custom$lazy$match$seq$str.str('true'), _alt$custom$lazy$match$seq$str.str('false'));

var json = lex(_alt$custom$lazy$match$seq$str.alt(keywords, number, string, array, object));

exports.json = json;
// Primitive types
// ---------------

var identifier = ident.as('identifier');
exports.identifier = identifier;
var variable = _alt$custom$lazy$match$seq$str.seq(angleL, ident.as('variable'), angleR);
exports.variable = variable;
var ref = _alt$custom$lazy$match$seq$str.seq(amp, _alt$custom$lazy$match$seq$str.match(/\d+/).as('ref'));

exports.ref = ref;
// Rules
// -----

var arg = _alt$custom$lazy$match$seq$str.alt(variable, ref, json.as('json'));
exports.arg = arg;
var call = _alt$custom$lazy$match$seq$str.seq(identifier, parenL, sep(arg, comma).as('args'), parenR);
exports.call = call;
var calls = _alt$custom$lazy$match$seq$str.seq(period, call).repeat(1).as('calls');

exports.calls = calls;
var field = _alt$custom$lazy$match$seq$str.lazy(function () {
  return _alt$custom$lazy$match$seq$str.seq(identifier, calls.maybe(), fields.maybe());
});
exports.field = field;
var fields = _alt$custom$lazy$match$seq$str.seq(braceL, sep(_alt$custom$lazy$match$seq$str.alt(field, ref), comma), braceR).as('fields');

exports.fields = fields;
var query = _alt$custom$lazy$match$seq$str.seq(call, fields);
exports.query = query;
var fragment = _alt$custom$lazy$match$seq$str.seq(ident.as('type'), fields);

exports.fragment = fragment;
var node = _alt$custom$lazy$match$seq$str.alt(query, fragment);

exports.node = node;
/**
 * Parses the given GraphQL expression and returns the resulting AST.
 */

exports['default'] = function (source) {
  return node.parse(String(source));
};